@@ -1,6 +1,6 @@
 //===----------------------------------------------------------------------===//
 //
-// Copyright (c) 2012, 2013, 2014, 2015 The University of Utah
+// Copyright (c) 2012, 2013, 2014, 2015, 2016 The University of Utah
 // All rights reserved.
 //
 // This file is distributed under the University of Illinois Open Source
@@ -21,7 +21,6 @@
 #include "TransformationManager.h"
 
 using namespace clang;
-using namespace llvm;
 
 static const char *DescriptionMsg =
 "Reduce pointer indirect level for a global/local variable. \
@@ -198,7 +197,8 @@ bool PointerLevelCollectionVisitor::VisitBinaryOperator(BinaryOperator *BO)
       dyn_cast<UnaryOperator>(Rhs) ||
       dyn_cast<ArraySubscriptExpr>(Rhs) ||
       dyn_cast<MemberExpr>(Rhs) ||
-      dyn_cast<IntegerLiteral>(Rhs))
+      dyn_cast<IntegerLiteral>(Rhs) ||
+      dyn_cast<CXXNewExpr>(Rhs))
     return true;
 
   const DeclaratorDecl *DD = ConsumerInstance->getRefDecl(Lhs);
@@ -560,11 +560,6 @@ const DeclaratorDecl *ReducePointerLevel::getRefDecl(const Expr *Exp)
   if (!UO)
     return NULL;
 
-  UnaryOperator::Opcode Op = UO->getOpcode();
-  (void)Op;
-  TransAssert(((Op == UO_Deref) || (Op == UO_AddrOf) ||
-              UO->isPrefix() || UO->isPostfix()) &&
-              "Invalid Unary Opcode!");
   const Expr *SubE = UO->getSubExpr();
   return getRefDecl(SubE);
 }
@@ -602,7 +597,8 @@ ReducePointerLevel::getCanonicalDeclaratorDecl(const Expr *E)
   if (const DeclRefExpr *DRE = dyn_cast<DeclRefExpr>(E)) {
     const ValueDecl *ValueD = DRE->getDecl();
     DD = dyn_cast<DeclaratorDecl>(ValueD);
-    TransAssert(DD && "Bad Declarator!");
+    if (!DD)
+      return NULL;
   }
   else if (const MemberExpr *ME = dyn_cast<MemberExpr>(E)) {
     ValueDecl *OrigDecl = ME->getMemberDecl();
