@@ -1,6 +1,6 @@
 //===----------------------------------------------------------------------===//
 //
-// Copyright (c) 2012, 2013, 2014, 2015 The University of Utah
+// Copyright (c) 2012, 2013, 2014, 2015, 2016, 2017 The University of Utah
 // All rights reserved.
 //
 // This file is distributed under the University of Illinois Open Source
@@ -269,7 +269,8 @@ const Expr *Transformation::getMemberExprBaseExprAndIdxs(
   while (ME) {
     ValueDecl *VD = ME->getMemberDecl();
     FieldDecl *FD = dyn_cast<FieldDecl>(VD);
-    TransAssert(FD && "Bad FD!\n");
+    if (!FD)
+      return NULL;
     unsigned int Idx = FD->getFieldIndex();
     Idxs.push_back(Idx);
 
@@ -288,11 +289,12 @@ const Expr *Transformation::getMemberExprBaseExprAndIdxs(
 bool Transformation::isCXXMemberExpr(const MemberExpr *ME)
 {
   const ValueDecl *VD = ME->getMemberDecl();
-  if (dyn_cast<CXXMethodDecl>(VD))
-    return true;
 
   const FieldDecl *FD = dyn_cast<FieldDecl>(VD);
-  TransAssert(FD && "Bad FieldDecl!");
+  // VD can be either CXXMethodDecl, EnumConstantDecl or
+  // VarDecl (static data member)
+  if (!FD)
+    return true;
   const CXXRecordDecl *CXXRD = dyn_cast<CXXRecordDecl>(FD->getParent());
   if (!CXXRD)
     return false;
@@ -307,6 +309,8 @@ const Expr *Transformation::getMemberExprElem(const MemberExpr *ME)
 
   IndexVector Idxs;
   const Expr *BaseE = getMemberExprBaseExprAndIdxs(ME, Idxs);
+  if (!BaseE)
+    return NULL;
   return getInitExprFromBase(BaseE, Idxs);
 }
 
@@ -340,13 +344,7 @@ unsigned int Transformation::getArrayDimensionAndTypes(
 
 const Type *Transformation::getArrayBaseElemType(const ArrayType *ArrayTy)
 {
-  const Type *ArrayElemTy = ArrayTy->getElementType().getTypePtr();
-  while (ArrayElemTy->isArrayType()) {
-    const ArrayType *AT = dyn_cast<ArrayType>(ArrayElemTy);
-    ArrayElemTy = AT->getElementType().getTypePtr();
-  }
-  TransAssert(ArrayElemTy && "Bad Array Element Type!");
-  return ArrayElemTy;
+  return ArrayTy->getBaseElementTypeUnsafe();
 }
 
 unsigned int Transformation::getConstArraySize(
@@ -664,6 +662,9 @@ const CXXRecordDecl *Transformation::getBaseDeclFromTemplateSpecializationType(
   TemplateName TplName = TSTy->getTemplateName();
   TemplateDecl *TplD = TplName.getAsTemplateDecl();
   TransAssert(TplD && "Invalid TemplateDecl!");
+  if (dyn_cast<TemplateTemplateParmDecl>(TplD)) {
+    return NULL;
+  }
   NamedDecl *ND = TplD->getTemplatedDecl();
   TransAssert(ND && "Invalid NamedDecl!");
 
@@ -737,6 +738,12 @@ const CXXRecordDecl *Transformation::getBaseDeclFromType(const Type *Ty)
     return getBaseDeclFromType(PT);
   }
 
+  case Type::Pointer: {
+    const PointerType *PT = dyn_cast<PointerType>(Ty);
+    const Type *PTy = PT->getPointeeType().getTypePtr();
+    return getBaseDeclFromType(PTy);
+  }
+
   case Type::SubstTemplateTypeParm: {
     const SubstTemplateTypeParmType *TP =
       dyn_cast<SubstTemplateTypeParmType>(Ty);
@@ -771,6 +778,8 @@ const CXXRecordDecl *Transformation::getBaseDeclFromType(const Type *Ty)
   case Type::FunctionNoProto:
   case Type::SubstTemplateTypeParmPack:
   case Type::PackExpansion:
+  case Type::Vector:
+  case Type::ExtVector:
   case Type::Builtin: // fall-through
     return NULL;
 
@@ -782,6 +791,16 @@ const CXXRecordDecl *Transformation::getBaseDeclFromType(const Type *Ty)
     return getBaseDeclFromType(AT);
   }
 
+  case Type::TypeOfExpr: {
+    const Expr *E = dyn_cast<TypeOfExprType>(Ty)->getUnderlyingExpr();
+    return getBaseDeclFromType(E->getType().getTypePtr());
+  }
+
+  case Type::TypeOf: {
+    return getBaseDeclFromType(
+      dyn_cast<TypeOfType>(Ty)->getUnderlyingType().getTypePtr());
+  }
+
   default:
     Base = Ty->getAsCXXRecordDecl();
     TransAssert(Base && "Bad base class type!");
